import java.util.*;
import java.io.*;

/**
 * DNF will translate arbitrary boolean expressions
 * to a canonical form (Disjunctive Normal Form).
 * @author Dimitris Michelinakis
 */
public class DNF {

    /**
     * main is the main method of the DNF compiler.
     * @param args String[] argument is any parameters that we might want to use in the future.
     * @author Dimitris Michelinakis
     */
    public static void main (String args[]) throws IOException {
        System.out.println("=[Boolean Expression Compiler - B.E.C.");
        /* Prepare our buffer to read the expressions from STDIN */
        Reader r = new BufferedReader(new InputStreamReader(System.in));
        /* Tokenize the input */
        StreamTokenizer st = new StreamTokenizer(r);
        /* Vector to store 'final' expressions */
        Vector exp=new Vector(10);
        String expression="";
        /* Reset the rules of the StreamTokenizer */
        st.resetSyntax();
        /* Can detect // comments */
        st.slashSlashComments(true);
        /* Can detect /* comments */
        st.slashStarComments(true);
        /* EOL is significant, since we parse multiple expressions */
        st.eolIsSignificant(false);
        /* Spaces are ignored */
        st.whitespaceChars(32,32);
        /* Tabs are ignored */
        st.whitespaceChars(9,9);
        /* Main loop of lexical analysis */
        int c=0;
        /* While this is not the end of file... */
        while(c!=st.TT_EOF) {
            /* Read next token */
            c=st.nextToken();
            /* Is this token a whitespace character? (0Dh0Ah=EOL) */
            if(c==13|c==st.TT_EOL&expression.equals("")==true)
                continue; /* Yes so skip the rest */
            /* Have we reached the end of our expression/line? */
            if(c==st.TT_EOL&expression.equals("")==false) {
                /* Yes, so add the whole expression in our Vector */
                exp.addElement(expression);
                /* Reset, and continue for more expressions */
                expression="";
                continue;
            }
            /* ReCheck: Have we reached the end of file? */
            if(c==st.TT_EOF)
                break; /* Yes so... get a break */
            /* Add more characters in the expression */
            expression=expression.concat(new Character(st.toString().charAt(7)).toString());
        }
        /* Enumerate our expressions */
        Enumeration enum=exp.elements();
        /* Scan all our expressions and create AST's */
        while(enum.hasMoreElements()) {
            /* Get the next expression */
            expression=(String)enum.nextElement();
            System.out.println();
            System.out.println("=[Expression :"+expression);
            System.out.print("=[Result     :");
            /* Parse the expression */
            Parser p=new Parser(expression);
            /* In case parsing failed, skip this expression */
            if(p.ok==false)
                continue;
            /* Create an AST tree based on the parsed expression */
            AbstractST parserAstTree=p.toTree();
            /* Print the final expression with expanded And/Or operators */
            System.out.println("=[Expanded   :"+parserAstTree.printTree());
            /* Convert the tree to Reverse Polish Notation */
            ReversePolishNotation myConverter = new ReversePolishNotation(parserAstTree.printTree());
            /* Print the Reverse Polish Notation */
            System.out.println("=[RevPolishN :"+myConverter);
            /* At this point we use the RevPolishN to generate
             combinations of the boolean expression */
        }
        System.out.println();
        System.out.println("=[The End");
    }

}
