/**
 * Parser will parse a boolean expression stored in
 * a String and will generate a parser tree.
 * @author Dimitris Michelinakis
 */
public class Parser {
    /** String to store the boolean expression */
    private String expression;
    /** Token poiner, poits at the current character possition */
    private int token=0;
    /** The parser tree */
    private AbstractST tree;
    /** If parsing worked ok */
    public boolean ok=true;

    /**
     * Parser is the constructor of a parsed boolean expression.
     * @param exp String argument is expression itself.
     * @author Dimitris Michelinakis
     */
    public Parser (String exp) {
        /* Remember the expression */
        expression=exp;
        /* Parse the exression and create a parser tree */
        tree=parse_S();
        if(ok==true)
            System.out.println("*true*");
    }

    /**
     * toTree is a method which returns the tree of the
     * parsed expression.
     * @return an <code>AbstractST</code> with the tree of the expression.
     * @author Dimitris Michelinakis
     */
    public AbstractST toTree () {
        return tree;
    }

    /**
     * parse_S is a method which represents the S variable
     * of the LL(1) grammar.
     * @return a <code>AbstractST</code> with the created node(s).
     * @author Dimitris Michelinakis
     */
    private AbstractST parse_S () {
        AbstractST left,right;
        /* Have we reached the end of the expression or an error? */
        if(token>expression.length()-1|ok==false)
            return null;
        switch(expression.charAt(token)) {
        case 'a': case 'b': case 'c': case 'd': case 'e': case 'f': case 'g':
        case 'h': case 'i': case 'j': case 'k': case 'l': case 'm': case 'n':
        case 'o': case 'p': case 'q': case 'r': case 's': case 't': case 'u':
        case 'v': case 'w': case 'x': case 'y': case 'z': case 'A': case 'B':
        case 'C': case 'D': case 'E': case 'F': case 'G': case 'H': case 'I':
        case 'J': case 'K': case 'L': case 'M': case 'N': case 'O': case 'P':
        case 'Q': case 'R': case 'S': case 'T': case 'U': case 'V': case 'W':
        case 'X': case 'Y': case 'Z':
        case '(':
            left=parse_E();
            right=parse_S2();
            return new AbstractST(left,right);
        default:
            System.out.println("*false*");
            ok=false;
        }
        return null;
    }

    /**
     * parse_E is a method which represents the E variable
     * of the LL(1) grammar.
     * @return an <code>AbstractST</code> with the created node(s).
     * @author Dimitris Michelinakis
     */
    private AbstractST parse_E () {
        AbstractST left,right;
        /* Have we reached the end of the expression or an error? */
        if(token>expression.length()-1|ok==false)
            return null;
        switch(expression.charAt(token)) {
        case 'a': case 'b': case 'c': case 'd': case 'e': case 'f': case 'g':
        case 'h': case 'i': case 'j': case 'k': case 'l': case 'm': case 'n':
        case 'o': case 'p': case 'q': case 'r': case 's': case 't': case 'u':
        case 'v': case 'w': case 'x': case 'y': case 'z': case 'A': case 'B':
        case 'C': case 'D': case 'E': case 'F': case 'G': case 'H': case 'I':
        case 'J': case 'K': case 'L': case 'M': case 'N': case 'O': case 'P':
        case 'Q': case 'R': case 'S': case 'T': case 'U': case 'V': case 'W':
        case 'X': case 'Y': case 'Z':
            left=new AbstractST(expression.charAt(token));
            token++;
            return left;
        case '(':
            token++;
            left=parse_S();
            token++;
            return left;
        default:
            System.out.println("*false*");
            ok=false;
        }
        return null;
    }

    /**
     * parse_S2 is a method which represents the S2 variable
     * of the LL(1) grammar.
     * @return an <code>AbstractST</code> with the created node(s).
     * @author Dimitris Michelinakis
     */
    private AbstractST parse_S2 () {
        AbstractST left,right;
        /* Have we reached the end of the expression or an error? */
        if(token>expression.length()-1|ok==false)
            return null;
        switch(expression.charAt(token)) {
        case 'a': case 'b': case 'c': case 'd': case 'e': case 'f': case 'g':
        case 'h': case 'i': case 'j': case 'k': case 'l': case 'm': case 'n':
        case 'o': case 'p': case 'q': case 'r': case 's': case 't': case 'u':
        case 'v': case 'w': case 'x': case 'y': case 'z': case 'A': case 'B':
        case 'C': case 'D': case 'E': case 'F': case 'G': case 'H': case 'I':
        case 'J': case 'K': case 'L': case 'M': case 'N': case 'O': case 'P':
        case 'Q': case 'R': case 'S': case 'T': case 'U': case 'V': case 'W':
        case 'X': case 'Y': case 'Z':
            left=new AbstractST('/');
            right=parse_S();
            if(right==null)
                return left;
            return new AbstractST(left,right);
        case '\'':
            left=new AbstractST(expression.charAt(token));
            token++;
            right=parse_P();
            if(right==null)
                return left;
            return new AbstractST(left,right);
        case '/':
            left=new AbstractST(expression.charAt(token));
            token++;
            token++;
            right=parse_S();
            if(right==null)
                return left;
            return new AbstractST(left,right);
        case '\\':
            left=new AbstractST(expression.charAt(token));
            token++;
            token++;
            right=parse_S();
            if(right==null)
                return left;
            return new AbstractST(left,right);
        case '(':
            left=new AbstractST('/');
            right=parse_S();
            if(right==null)
                return left;
            return new AbstractST(left,right);
        case ')':
            return null;
        default:
            System.out.println("*false*");
            ok=false;
        }
        return null;
    }

    /**
     * parse_P is a method which represents the P variable
     * of the LL(1) grammar.
     * @return an <code>AbstractST</code> with the created node(s).
     * @author Dimitris Michelinakis
     */
    private AbstractST parse_P () {
        if(token>expression.length()-1|ok==false)
            return null;
        switch(expression.charAt(token)) {
        case ')':
            return null;
        }
        return parse_S2();
    }

}
