import java.util.*;

/**
 * ReversePolishNotation will take a boolean expression
 * and return it in Reverse Polish Notation.
 */
public class ReversePolishNotation {
    /** String to store the input expression */
    private String inputString;
    /** String to store the reversed expression */
    private String outputString="";

    /**
     * ReversePolishNotation is the constructor that
     * takes a String with the boolean expression in infix
     * notation and stores the expression in Reverse Polish
     * Notation in the outputString.
     * @param s String argument is infix expression.
     * @author Dimitris Michelinakis
     */
    public ReversePolishNotation(String s) {
        /* Convert '/\' to single character '/' */
        int indx=s.indexOf("/\\");
        while(indx>=0) {
            /* Remove extra character */
            s=new StringBuffer(s).deleteCharAt(indx+1).toString();
            /* Search for more matches */
            indx=s.indexOf("/\\",indx+1);
        }
        /* Convert '\/' to single character '\' */
        indx=s.indexOf("\\/");
        while(indx>=0) {
            /* Remove extra character */
            s=new StringBuffer(s).deleteCharAt(indx+1).toString();
            /* Search for more matches */
            indx=s.indexOf("\\/",indx+1);
        }
        /* Store the infix expression */
        inputString="["+s+"]";
        /* Convert and store the expression */
        this.convert();
    }

    /**
     * precedence is a method which returns an int, representing
     * the higher or lower precedence of the operator/letter.
     * @param s String argument is operator/letter.
     * @return an <code>int</code> which is the level of precedence.
     * @author Dimitris Michelinakis
     */
    private int precedence(String s) {
        /* Define the base leve of precedence */
        int operatorPriority = -1;
        /* Define precedence based on our table:
           Operator Precedence
           (        1
           \/       2
           /\       3
           '        4
        */
        if (s.equals("["))
            operatorPriority = 0;
        else if (s.equals("("))
            operatorPriority = 1;
        else if (s.equals("\\"))
            operatorPriority = 2;
        else if (s.equals("/"))
            operatorPriority = 3;
        else if (s.equals("'"))
            operatorPriority = 4;
        return(operatorPriority);
    }

    /**
     * convert is the private method that does the conversion
     * between infix and reverse polish notation.
     */
    private void convert() {
        /* Create the stack used for conversion */
        Stack myStack=new Stack();
        /* Our lookahead element */
        String nextCharacter;
        /* Last pushed element */
        String lookAhead;
        /* While there are infix characters... */
        for(int i=0;i<=inputString.length()-1;i++) {
            nextCharacter=inputString.substring(i,i+1);
            if(nextCharacter.equals(")")) {
                /* If the next character is ) */
                /* do a POP */
                lookAhead=(String)myStack.pop();
                /* While not a ( */
                while(!lookAhead.equals("(")) {
                    /* Do a STORE */
                    outputString+=lookAhead;
                    /* Do a POP */
                    lookAhead=(String)myStack.pop();
                }
            } else if(nextCharacter.equals("]")) {
                /* If the next character is ] */
                /* Do a POP */
                lookAhead=(String)myStack.pop();
                /* While not a [ */
                while(!lookAhead.equals("[")) {
                    /* Do a STORE */
                    outputString+=lookAhead;
                    /* Do a POP */
                    lookAhead=(String)myStack.pop();
                }
            } else if(nextCharacter.equals("(")||nextCharacter.equals("["))
                /* If next character is ( or [ */
                /* Do a PUSH */
                myStack.push(nextCharacter);
            else if(nextCharacter.equals("\\")||nextCharacter.equals("/")||nextCharacter.equals("'")) {
                /* If next character is \ or / or ' */
                /* Do a PEEK */
                lookAhead=(String)myStack.peek();
                /* While precedence of next character is <=
                 of precedence of top of Stack */
                while(precedence(nextCharacter)<=precedence(lookAhead)) {
                    /* Do a POP */
                    lookAhead=(String)myStack.pop();
                    outputString+=lookAhead;
                    /* Do a PEEK */
                    lookAhead=(String)myStack.peek();
                }
                /* Do a PUSH */
                myStack.push(nextCharacter);
            } else /* Next character in buffer */
                outputString+=nextCharacter;
        }
    }

    /**
     * toString is a method which returns the string representation
     * of the boolean expression in Reverse Polish Notation.
     * @return a <code>String</code> which is a boolean expression in RPN.
     * @author Dimitris Michelinakis
     */
    public String toString() {
        return outputString;
    }

}
